//+------------------------------------------------------------------+
//|                                                     EnvXover.mq4 |
//|                                                      Colin Reeve |
//|                                             midworld08@gmail.com |
//+------------------------------------------------------------------+
#property copyright "Colin Reeve"
#property link      "midworld08@gmail.com"

extern double  LotSize = 0.1,
               StopLoss = 50,
               TakeProfit = 0,
               Envelope_M30 = 0.0015,
               Envelope_H1 = 0.0025;
extern int     Slippage = 5,
               MagicNumber = 164,
               MAPeriod = 5;
//Global Variables
int      BuyTicket,
         SellTicket,
         UseSlippage;
double   UsePoint;

int init()
{
   UsePoint = PipPoint(Symbol());                  //figure out how if the broker is a 4 or 5 digit broker
   UseSlippage = GetSlippage(Symbol(),Slippage);   //calculate the slippage based on whether the broker is 4/5 digit 
   BuyTicket=0;
   SellTicket=0;
   return(0);
}

int deinit()
  {
   return(0);
  }

int start()
{
string   CommentString;
//first check to see if in correct time zones, must be M30 or H1
if ((Period()== PERIOD_M30) || (Period()== PERIOD_H1))
   {
      CommentString = "Snap Back Active";
//fetch the MA and work out the envelope values
      double   MAValue = iMA(NULL,0,MAPeriod,0,MODE_EMA,0,0);
      double   UpperMAEnvelope, LowerMAEnvelope;
      if (Period() == PERIOD_M30)
      {
         UpperMAEnvelope = MAValue + (MAValue * Envelope_M30);
         LowerMAEnvelope = MAValue - (MAValue * Envelope_M30);
         CommentString = CommentString + " Period = 30 ";
      }
      if (Period() == PERIOD_H1)
      {
         UpperMAEnvelope = MAValue + (MAValue * Envelope_H1);
         LowerMAEnvelope = MAValue - (MAValue * Envelope_H1);
         CommentString = CommentString + " Period = 60 ";
      }
      Comment (CommentString, "\nMA Value ", MAValue, "\nUpper Env ", UpperMAEnvelope, "\nLower Env ", LowerMAEnvelope,
         "\nSellTicket ", SellTicket, "\nBuy Ticket ", BuyTicket);
      
      if ((SellTicket == 0) && (BuyTicket == 0)) //check if inside a trade, only allowed one open
      {
         if (Bid > UpperMAEnvelope) //go short as we have passed the upper Envelope
         {
            if (StopLoss > 0) double SellStopLoss = Ask + (StopLoss * UsePoint);    //work out the stop loss
            if (TakeProfit > 0) double SellTP = Bid - (TakeProfit * UsePoint);      //work out the take profit
            SellTicket = OrderSend(Symbol(), OP_SELL, LotSize, Bid, UseSlippage, SellStopLoss, 
               SellTP, "Sell Order", MagicNumber,0, Red);
            if (SellTicket < 0)
            {
               Print("OrderSend failed with error #",GetLastError());
               SellTicket=0;
            }
         }
         if (Ask < LowerMAEnvelope) //go long as we have passed the upper Envelope
         {
            if (StopLoss > 0) double BuyStopLoss = Bid - (StopLoss * UsePoint);    //work out the stop loss
            if (TakeProfit > 0) double BuyTP = Ask + (TakeProfit * UsePoint);      //work out the take profit
            BuyTicket = OrderSend(Symbol(), OP_BUY, LotSize, Ask, UseSlippage, BuyStopLoss, 
               BuyTP, "Buy Order", MagicNumber,0, Green);
            if (BuyTicket < 0)
            {
               Print("OrderSend failed with error #",GetLastError());
               BuyTicket=0;
            }
         }
      }
    //time to see if in a trade and whether exit condition has been met
      if ((SellTicket > 0) && (Ask <= MAValue))      //if its lower than the EMA exit position
      {
         if (OrderSelect(SellTicket,SELECT_BY_TICKET))
         {
            if (!OrderClose(SellTicket, OrderLots(), Ask, UseSlippage, Red))
               Print("Failed Close of order error :", GetLastError());
         }
         else
            Print("Failed Select of order error :", GetLastError());
         SellTicket=0;
      }
      if ((BuyTicket > 0) && (Bid >= MAValue))      //if its higher than the EMA exit position
      {
         if (OrderSelect(BuyTicket,SELECT_BY_TICKET))
         {
            if (!OrderClose(BuyTicket, OrderLots(), Bid, UseSlippage, Green))
               Print("Failed Close of order error :", GetLastError());
         }
         else
            Print("Failed Select of order error :", GetLastError());
         BuyTicket=0;
      }
   }
   else
         Comment("Snap Back not active, switch to M30 or H1");
   return(0);
}
 
//Calculate correct pip values for a 5 digit or 4 digit broker
double PipPoint(string Currency)
{
   int CalcDigits = MarketInfo(Currency, MODE_DIGITS);
   double CalcPoint;
   if (CalcDigits == 2 || CalcDigits == 3) CalcPoint = 0.01;
   if (CalcDigits == 4 || CalcDigits == 5) CalcPoint = 0.0001;
   return(CalcPoint);
}

//Work out the correct amount of slippage depending on the broker digits 
int GetSlippage(string Currency, int SlippagePips)
{
   int CalcDigits = MarketInfo(Currency, MODE_DIGITS);
   double CalcSlippage;
   if (CalcDigits == 2 || CalcDigits == 4) CalcSlippage = SlippagePips;
   if (CalcDigits == 3 || CalcDigits == 5) CalcSlippage = SlippagePips * 10;
   return(CalcSlippage);
}----+